<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Helper;

use DOMDocument;
use Magento\Framework\ObjectManagerInterface;
use Magento\PageBuilder\Model\Dom\Element;
use Magento\PageBuilder\Model\Dom\HtmlDocumentFactory;
use Sabberworm\CSS\Parser;

use function class_exists;

/**
 *
 */
class PageBuilderConverter
{
    /**
     * @var HtmlDocumentFactory | null
     */
    private $documentFactory;

    /**
     * @param ObjectManagerInterface $objectManager
     */
    public function __construct(ObjectManagerInterface $objectManager) {
        if (class_exists(HtmlDocumentFactory::class)) {
            $this->documentFactory = $objectManager->get(HtmlDocumentFactory::class);
        }
    }

    /**
     * @param $data
     * @return array|mixed|string|string[]|null
     * @throws \Sabberworm\CSS\Parsing\SourceException
     */
    public function convert($data)
    {
        if(empty($data)) {
            return $data;
        }

        if (!$this->documentFactory || !method_exists(Element::class, 'getOriginalElement')) {
            return $data;
        }

        $doc = new DOMDocument('1.0', 'UTF-8');
        $doc->loadHTML($data);
        $doc->normalizeDocument();

        $document = $this->documentFactory->create(['document' => $doc->saveHTML()]);
        $body = $document->getContents();
        $styles = $document->querySelectorAll('style');

        $stylesMap = [];

        foreach ($styles as $style) {
            foreach ($this->getStyles($style->getOriginalElement()->nodeValue) as $id => $value) {
                if (isset($stylesMap[$id])) {
                    $stylesMap[$id] .= $value;
                    continue;
                }

                $stylesMap[$id] = $value;
            }
        }

        if (empty($stylesMap)) {
            return $data;
        }

        $toSearch = implode('|', array_keys($stylesMap));
        preg_match('/<body>(.*)<\/body>/s', $body, $matches);

        return preg_replace_callback(
            "/data-pb-style=\"($toSearch)\"/m",
            static function ($matches) use ($stylesMap) {
                return 'style="' . trim($stylesMap[$matches[1]], '{}') . '"';
            },
            $matches[1]
        );
    }

    /**
     * @param string $css
     * @return array
     * @throws \Sabberworm\CSS\Parsing\SourceException
     */
    public function getStyles(string $css): array
    {
        $parser = new Parser($css);
        $cssDocument = $parser->parse();

        $lookup = [];

        foreach ($cssDocument->getAllDeclarationBlocks() as $block) {

            $d = array_reduce($block->getRules(), static function ($carry, $a) {
                return (string)$carry . $a;
            }, '');

            $d = trim($d, '"');
            foreach ($block->getSelectors() as $selector) {
                $selectorName = $selector->getSelector();
                $output = [];
                preg_match('/#html-body\s+\[data-pb-style=(.*)\]/', $selectorName, $output);

                $selectorId = trim($output[1], '"');

                if (!isset($lookup[$selectorId])) {
                    $lookup[$selectorId] = $d;
                    continue;
                }

                $lookup[$selectorId] .= $d;
            }
        }

        return $lookup;
    }
}
